
#include "comb/composition-nz.h"

#include "comb/comb-print.h"
#include "bits/printbin.h"
#include "demo/nextarg.h"

#include "aux0/sign.h" // Abs() (with OEIS counting)

#include "jjassert.h"

#include "fxttypes.h"
#include "fxtio.h"


//% Compositions of n into nonzero parts.

//#define TIMING // uncomment to disable printing

//#define OEIS

int
main(int argc, char **argv)
{
    ulong n = 6;
    NXARG(n, "Compositions of n (n>=1)");

    composition_nz P(n);
    P.first();

    ulong ct = 0;

#ifdef TIMING
    do { ++ct; }  while  ( P.next() <= n );
#else

#ifdef OEIS
    ulong * st = new ulong[n+1];  // statistics
    for (ulong j=0; j<n; ++j)  st[j] = 0;
#endif // OEIS

    const ulong *x = P.data();
    ulong q = 1;  // number of parts
    do
    {
#ifdef OEIS
        // OEIS sequences:
//        {  bool qq = true;
//            if ( x[0]!=1 || x[q-1]!=1 )  qq = false;
//            else
//            {
//                for (ulong j=1; j<q; ++j)
//                    if ( Abs<long>((long)(x[j]-x[j-1])) > 1 ) { qq=false; break; }
//            }
//            if ( !qq )  continue;  // (sand piles by height)
//
//            ulong h = 0; // A187081: height
//            for (ulong j=0; j<q; ++j)  if ( h<x[j] )  h=x[j];  // find max in x[]
//            st[h] += 1;
//            cout << h << "  ";
//        }
        {  bool qq = true;
            if ( x[0]!=1 )  qq = false;
            else
            {
                for (ulong j=1; j<q; ++j)
                    if ( (long)(x[j]-x[j-1]) > 1 ) { qq=false; break; }
            }
            if ( !qq )  continue;  // A005169 (fountains of n coins)

            ulong h = 0; // height: A187080
            for (ulong j=0; j<q; ++j)  if ( h<x[j] )  h=x[j];  // find max in x[]
            st[h] += 1;
            cout << h << "  ";

//            st[q] += 1;  // length of base: A047998
        }


#endif // OEIS

        ++ct;
        cout << setw(4) << ct << ":";
        print_vec("    ", x, q, true);
//        cout << "  " << q;
        cout << endl;

    }
    while ( (q=P.next()) <= n );
#endif


#ifdef OEIS
    cout << "  stats:  ";
    for (ulong j=0; j<=n; ++j)  cout << st[j] << " ";
    cout << endl;
    delete [] st;
#endif // OEIS

    cout << "  ct=" << ct << endl;

    return 0;
}
// -------------------------


/*
Timing:

 time ./bin 31
arg 1: 31 == n  [Compositions of n (n>=1)]  default=6
  ct=1073741824
./bin 31  4.77s user 0.00s system 99% cpu 4.771 total
 ==> 1073741824/4.77 == 225,103,107 compositions per second

*/

/*
Sand piles by height:
 for n in $(seq 1 20); do ./bin $n | g stats ; done | sed 's/stats://g;'
    0 1
    0 1 0
    0 1 0 0
    0 1 1 0 0
    0 1 2 0 0 0
    0 1 4 0 0 0 0
    0 1 7 0 0 0 0 0
    0 1 12 0 0 0 0 0 0
    0 1 20 1 0 0 0 0 0 0
    0 1 33 2 0 0 0 0 0 0 0
    0 1 54 5 0 0 0 0 0 0 0 0
    0 1 88 11 0 0 0 0 0 0 0 0 0
    0 1 143 22 0 0 0 0 0 0 0 0 0 0
    0 1 232 44 0 0 0 0 0 0 0 0 0 0 0
    0 1 376 84 0 0 0 0 0 0 0 0 0 0 0 0
    0 1 609 158 1 0 0 0 0 0 0 0 0 0 0 0 0
    0 1 986 293 2 0 0 0 0 0 0 0 0 0 0 0 0 0
    0 1 1596 535 5 0 0 0 0 0 0 0 0 0 0 0 0 0 0
    0 1 2583 969 12 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
    0 1 4180 1739 25 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0

column 2: Fibonacci - 1

 */


/// Emacs:
/// Local Variables:
/// MyRelDir: "demo/comb"
/// makefile-dir: "../../"
/// make-target: "1demo DSRC=demo/comb/composition-nz-demo.cc"
/// make-target2: "1demo DSRC=demo/comb/composition-nz-demo.cc DEMOFLAGS=-DTIMING"
/// End:
